@@ -1,11 +1,13 @@
 """Builder superclass for all builders."""
 
+from __future__ import annotations
+
 import os
 import re
 import warnings
 from datetime import datetime, timezone
 from os import path
-from typing import TYPE_CHECKING, Callable, Generator, List, NamedTuple, Tuple, Union
+from typing import TYPE_CHECKING, Callable, Generator, NamedTuple
 
 import babel.dates
 from babel.messages.mofile import write_mo
@@ -71,7 +73,7 @@ class CatalogInfo(LocaleFileInfoBase):
 class CatalogRepository:
     """A repository for message catalogs."""
 
-    def __init__(self, basedir: str, locale_dirs: List[str],
+    def __init__(self, basedir: str, locale_dirs: list[str],
                  language: str, encoding: str) -> None:
         self.basedir = basedir
         self._locale_dirs = locale_dirs
@@ -92,7 +94,7 @@ class CatalogRepository:
                 logger.verbose(__('locale_dir %s does not exists'), locale_path)
 
     @property
-    def pofiles(self) -> Generator[Tuple[str, str], None, None]:
+    def pofiles(self) -> Generator[tuple[str, str], None, None]:
         for locale_dir in self.locale_dirs:
             basedir = path.join(locale_dir, self.language, 'LC_MESSAGES')
             for root, dirnames, filenames in os.walk(basedir):
@@ -113,7 +115,7 @@ class CatalogRepository:
             yield CatalogInfo(basedir, domain, self.encoding)
 
 
-def docname_to_domain(docname: str, compaction: Union[bool, str]) -> str:
+def docname_to_domain(docname: str, compaction: bool | str) -> str:
     """Convert docname to domain for catalogs."""
     if isinstance(compaction, str):
         return compaction
@@ -190,7 +192,9 @@ def babel_format_date(date: datetime, format: str, locale: str,
         return format
 
 
-def format_date(format: str, date: datetime = None, language: str = None) -> str:
+def format_date(
+    format: str, date: datetime | None = None, language: str | None = None
+) -> str:
     if date is None:
         # If time is not specified, try to use $SOURCE_DATE_EPOCH variable
         # See https://wiki.debian.org/ReproducibleBuilds/TimestampsProposal
@@ -229,9 +233,9 @@ def format_date(format: str, date: datetime = None, language: str = None) -> str
     return "".join(result)
 
 
-def get_image_filename_for_language(filename: str, env: "BuildEnvironment") -> str:
+def get_image_filename_for_language(filename: str, env: BuildEnvironment) -> str:
     filename_format = env.config.figure_language_filename
-    d = dict()
+    d = {}
     d['root'], d['ext'] = path.splitext(filename)
     dirname = path.dirname(d['root'])
     if dirname and not dirname.endswith(path.sep):
@@ -249,7 +253,7 @@ def get_image_filename_for_language(filename: str, env: "BuildEnvironment") -> s
         raise SphinxError('Invalid figure_language_filename: %r' % exc) from exc
 
 
-def search_image_for_language(filename: str, env: "BuildEnvironment") -> str:
+def search_image_for_language(filename: str, env: BuildEnvironment) -> str:
     translated = get_image_filename_for_language(filename, env)
     _, abspath = env.relfn2path(translated)
     if path.exists(abspath):
